import pytest


pytest_plugins = "pytester"


@pytest.fixture(autouse=True)
def _divert_atexit(request, monkeypatch):
    import atexit
    l = []

    def finish():
        while l:
            l.pop()()

    monkeypatch.setattr(atexit, "register", l.append)
    request.addfinalizer(finish)



from setuptools import setup

setup(
    name="pytest-forked",
    use_scm_version=True,
    description='run tests in isolated forked subprocesses',
    long_description=open('README.rst').read(),
    license='MIT',
    author='pytest-dev',
    author_email='pytest-dev@python.org',
    url='https://github.com/pytest-dev/pytest-forked',
    platforms=['linux', 'osx'],
    packages=['pytest_forked'],
    package_dir={'': 'src'},
    entry_points={
        'pytest11': [
            'pytest_forked = pytest_forked',
        ],
    },
    zip_safe=False,
    install_requires=['pytest>=2.6.0'],
    setup_requires=['setuptools_scm'],
    classifiers=[
        'Development Status :: 7 - Inactive',
        'Framework :: Pytest',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Topic :: Software Development :: Testing',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Utilities',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
)



import pytest
import os

needsfork = pytest.mark.skipif(not hasattr(os, "fork"),
                               reason="os.fork required")


@needsfork
def test_functional_boxed(testdir):
    p1 = testdir.makepyfile("""
        import os
        def test_function():
            os.kill(os.getpid(), 15)
    """)
    result = testdir.runpytest(p1, "--forked")
    result.stdout.fnmatch_lines([
        "*CRASHED*",
        "*1 failed*"
    ])


@needsfork
@pytest.mark.parametrize("capmode", [
    "no",
    pytest.mark.xfail("sys", reason="capture cleanup needed"),
    pytest.mark.xfail("fd", reason="capture cleanup needed")])
def test_functional_boxed_capturing(testdir, capmode):
    p1 = testdir.makepyfile("""
        import os
        import sys
        def test_function():
            sys.stdout.write("hello\\n")
            sys.stderr.write("world\\n")
            os.kill(os.getpid(), 15)
    """)
    result = testdir.runpytest(p1, "--forked", "--capture=%s" % capmode)
    result.stdout.fnmatch_lines("""
        *CRASHED*
        *stdout*
        hello
        *stderr*
        world
        *1 failed*
""")


def test_is_not_boxed_by_default(testdir):
    config = testdir.parseconfig(testdir.tmpdir)
    assert not config.option.forked
